@@ -1,7 +1,7 @@
 /***************************************************************************
                           defaultplugin.cpp  -  description
                              -------------------
-    begin                : 日 10月 26 2003
+    begin                :  10 26 2003
     copyright            : (C) 2003 by Sheldon Lee Wen
     email                : leewsb@hotmail.com
  ***************************************************************************/
@@ -41,28 +41,50 @@ using namespace lineak_core_functions;
 #include "defaultplugin.h"
 
 /** These are required */
-#define NUM_MACROS 12 
+#define NUM_MACROS 16 
+#define INPUT 1
+#define ROOT 2
+#define POINTER 3
 
-string default_symbols[NUM_MACROS] = { "EAK_EJECT", "EAK_OPEN_TRAY_SCSI", "EAK_OPEN_TRAY", "EAK_CLOSE_TRAY", "EAK_VOLUP", "EAK_VOLDOWN", "EAK_PCM_UP", "EAK_PCM_DOWN", "EAK_MUTE", "EAK_PCM_MUTE", "EAK_SLEEP", "EAK_SYM" };
+string default_symbols[NUM_MACROS] = { "EAK_EJECT", "EAK_OPEN_TRAY_SCSI", "EAK_OPEN_TRAY", "EAK_CLOSE_TRAY", "EAK_VOLUP", "EAK_VOLDOWN", "EAK_PCM_UP", "EAK_PCM_DOWN", "EAK_MUTE", "EAK_PCM_MUTE", "EAK_SLEEP", "EAK_SENDKEYS", "EAK_SENDKEYS_ROOT", "EAK_SENDKEYS_WINDOW", "EAK_SYM", "EAK_SCREEN_LOCK" };
 macro_info* default_macinfo = NULL;
 identifier_info* idinfo = NULL;
 LConfig* myConfig = NULL;
 bool verbose = false;
+bool very_verbose = false;
 bool enable;
 bool trayopen = false;
 PluginManager* plugins = NULL;
 string dname = "";
 displayCtrl* default_Display = NULL;
-//Xmgr *mX = NULL;
+
+// Needed for the XKEYEVENT macros
+// 
+/*
+Display *dpy;
+int scr,x,y;
+Window win, ret_win;
+unsigned int width, height;
+unsigned int border_width, depth;
+*/
 
 extern "C" int initialize(init_info init) {
         verbose = init.verbose;
+	very_verbose = init.very_verbose;
 	enable = init.global_enable;
 	myConfig = init.config;
         plugins = init.plugins;
-	msg("Registered");
-        //if (mX == NULL)
-        //   mX = new Xmgr();
+
+/*        dpy = XOpenDisplay("");
+        if (!dpy) {
+           error("Could not open the display.");
+	   return false;
+        }
+	scr = DefaultScreen(dpy);
+	win = RootWindow(dpy, scr);
+        XGetGeometry(dpy, win, &ret_win, &x, &y, &width, &height, &border_width, &depth);
+*/	
+	vmsg("Registered");
 
         return true;
 }
@@ -98,7 +120,7 @@ extern "C" int exec(LObject* imyKey,XEvent xev) {
 	}
 	        /* lookup the key in our EAKeylist */
         if (!command.isEmpty())    {
-                if (verbose) cout << "enable = " << enable << endl;
+                if (very_verbose) cout << "enable = " << enable << endl;
                 if (enable) {
                         if (verbose) cout << "... that's the " <<  imyKey->getName() << " key" << endl;
                         /* check if the key has to run a SPECIAL command */
@@ -125,8 +147,13 @@ extern "C" int exec(LObject* imyKey,XEvent xev) {
                                 macroEAK_MUTE(command,USE_PCM);
                         } else if (command.getMacroType() == "EAK_SLEEP" ) { /* Sleep (not implemented yet) */
                                 macroEAK_SLEEP(command);
-                        } else if (command.getMacroType() == "EAK_SYM" ) { /* Key SYM handling */
-                                macroEAK_SYM(imyKey, command);
+                        } else if (command.getMacroType() == "EAK_SENDKEYS" ||
+				   command.getMacroType() == "EAK_SYM"  ) { /* Key SYM handling */
+                                macroEAK_SYM(imyKey, command, INPUT);
+			} else if (command.getMacroType() == "EAK_SENDKEYS_ROOT" ) { /* Key event handling */
+				macroEAK_SYM(imyKey, command, ROOT);
+			} else if (command.getMacroType() == "EAK_SENDKEYS_WINDOW" ) { /* Key event handling */
+				macroEAK_SYM(imyKey, command, POINTER);
                         } else if (command.isMacro()) { /** If we are a macro, but we can't find one that matches, return false. */
                                 return false;
                         } else if (command.getCommand() == snull) { /* no command defined */
@@ -169,10 +196,129 @@ extern "C" void initialize_display(displayCtrl *imyDisplay) {
         if (verbose) cout << "Initializing display!" << endl;
         default_Display = imyDisplay;
 }
-void macroEAK_SYM(LObject *obj, LCommand &command) {
-const vector<string>& args = command.getArgs();
+
+void macroEAK_XSENDKEYS(LObject *obj, LCommand &command) {
+/*
+   const vector<string>& args = command.getArgs();
+   string macro = command.getMacroType();
+
+   if (args.size() == 1) {
+
+	   string comm = "bash -c \"xsendkeys ";
+	   comm += "\"";
+	   comm += args[0];
+	   comm += "\"\"";
+	   if (!fork()) {
+		   //comm += " &";
+		   if (verbose) cout << "... running " << comm << endl;
+                   if (verbose) cout << "... displaying " << dname << endl;
+                   system(comm.c_str());
+                   exit(true);
+           }
+	   
+      //
+      Display *dpy;
+      int scr;
+      Window win;
+      unsigned int width, height;
+
+      Window ret_win;
+      int x, y;
+      unsigned int border_width, depth;
+
+      dpy = XOpenDisplay("");
+      if (!dpy) {
+  	  error("Cannot connect to display!");
+      }
+      scr = DefaultScreen(dpy);
+      win = RootWindow(dpy, scr);
+      XGetGeometry(dpy, win, &ret_win, &x, &y, &width, &height, &border_width, &depth);
+
+      int keycode;
+      vector<string> elements;
+      vector<int>codes;
+      KeySym mykeysym;
+      KeyCode xkeycode;
+
+      int index = 0;
+      string parsed = "";
+      string symname = lineak_util_functions::strip_space(args[0]);
+      string smodifiers = "";
+    
+      while (symname.find('+') != string::npos) {
+              index = symname.find('+');
+              smodifiers = symname.substr(index+1,symname.size()-(index+1));
+              parsed = symname.substr(0,index);
+              symname = smodifiers;
+              if ( parsed == "control" )
+                      parsed = "Control_L";
+              if ( parsed == "shift" )
+                      parsed = "Shift_L";
+              if ( parsed == "mod1" || parsed == "alt" )
+                      parsed = "Alt_L";
+              cout << "element = " << parsed << endl;
+              elements.push_back(parsed);
+              cout << "remainder = " << smodifiers << endl;
+       }
+       elements.push_back(smodifiers);
+
+       for (vector<string>::const_iterator it = elements.begin(); it != elements.end(); it++) {
+               mykeysym = XStringToKeysym(it->c_str());
+               xkeycode = XKeysymToKeycode(dpy, mykeysym);
+               cout << *it << " has code of " << (int)xkeycode << endl;
+               codes.push_back((int)xkeycode);
+       }
+
+       cout << "vector has " << codes.size() << " elements: { ";
+       for (vector<int>::const_iterator it = codes.begin(); it != codes.end(); it++)
+                       cout << *it << " ";
+       cout << "} " << endl;
+
+       for (vector<int>::const_iterator it = codes.begin(); it != codes.end(); it++) {
+               keycode = *it;
+               if ( !XTestFakeKeyEvent(dpy, keycode, 1, 0) )
+		       cout << "Could not send event!" << endl;
+               cout << "sending key down: " << keycode << endl;
+               XSync(dpy, 1);
+       }
+       vector<int>::const_iterator it = codes.end();
+
+       do {
+            it--;
+            keycode = *it;
+            XTestFakeKeyEvent(dpy, keycode, 0, 0);
+            cout << "sending key up: " << keycode << endl;
+            XSync(dpy, 1);
+       }
+       while (it != codes.begin());
+       //
+   }
+*/
+}
+
+void macroEAK_SYM(LObject *obj, LCommand &command, int type) {
+   const vector<string>& args = command.getArgs();
    string macro = command.getMacroType();
    if (args.size() == 1) {
+      bool root, mouse, input;
+      root = mouse = input = false;
+      /** Determine the type */
+      switch (type) {
+	   case INPUT: 
+		   input = true;
+		   vmsg("Type is input");
+		   break;
+	   case ROOT: 
+		   root = true;
+		   vmsg("Type is root");
+		   break;
+	   case POINTER: 
+		   mouse = true;
+		   vmsg("Type is pointer");
+		   break;
+	   default:
+		   input = true;
+      }
       /** Expecting a KEYSYM string. */
       /** could be something like: "control+alt+S" or "shift+F1" or just "A" 
        *  This is the keysym we want to emit. */
@@ -195,20 +341,20 @@ const vector<string>& args = command.getArgs();
       KeySym *keysym;
       char *name;
       
-      /** parse the symname for modifiers. Strip off the modifier portion. */
+      // parse the symname for modifiers. Strip off the modifier portion. 
       if (symname.find('+') != string::npos) {
           index = symname.rfind('+');
           parsed = symname.substr(index+1,symname.size()-(index+1));
           smodifiers = symname.substr(0,index);
-          cout << "modifiers = " << smodifiers << endl;
-          cout << "symname = " << parsed << endl;
+          msg("modifiers = " + smodifiers);
+          msg("symname = " + parsed);
           modifiers = lineak_core_functions::getModifierNumericValue(smodifiers);
-          cout << "modifiers numerical = " << modifiers << endl;
+          if (verbose) cout << "modifiers numerical = " << modifiers << endl;
       }
       else {
           parsed = symname;
-          cout << "modifiers = none" << endl;
-	  cout << "symname = " << parsed << endl;
+          msg("modifiers = none");
+	  msg("symname = " + parsed);
       }
 
       name = (char *) malloc (parsed.size()+1);
@@ -238,16 +384,40 @@ const vector<string>& args = command.getArgs();
       }
 
       Display *dpy = NULL;
-      if ((dpy = XOpenDisplay(NULL)) == NULL)
-         cerr << "Could not open the display." << endl;
+      if ((dpy = XOpenDisplay("")) == NULL)
+         error("Could not open the display.");
 
       XKeyEvent xev;
         
       // Get InputFocus window. Of some reason "InputFocus" dosn't work 
-      Window w;
+      Window w, win;
       int revertTo;
-      XGetInputFocus(dpy, &w, &revertTo);
-  
+      int pointer_x, pointer_y, root_x_return, root_y_return, win_x_return, win_y_return;
+
+      pointer_x = pointer_y = win_x_return = win_y_return = 1;
+      root_x_return = root_y_return = 1;
+
+      if (root) {
+	 vmsg("Doing root window.");
+         int scr;
+         unsigned int width, height, border_width, depth;
+	 scr = DefaultScreen(dpy);
+	 win = RootWindow(dpy, scr);
+	 XGetGeometry(dpy, win, &w, &win_x_return, &win_y_return, &width, &height, &border_width, &depth);
+      }
+      else if (input) {
+	 vmsg("Doing input window.");
+         XGetInputFocus(dpy, &w, &revertTo);
+      }
+      else if (mouse) {
+	 vmsg("Doing mouse input window.");
+	 Window child;
+	 unsigned int mask_return;
+	 XGetInputFocus(dpy, &win, &revertTo);
+	 XQueryPointer(dpy, win, &w, &child, &root_x_return, &root_y_return, &win_x_return, &win_y_return, &mask_return);
+	 w = child;
+      }
+
       xev.display = dpy; 
       xev.root = DefaultRootWindow(dpy);
       xev.window = w;
@@ -255,23 +425,24 @@ const vector<string>& args = command.getArgs();
       xev.subwindow = None;
       xev.type = KeyPress;
       xev.keycode = XKeysymToKeycode(dpy, mykeysym);
-      //xev.keycode = 56;
-      //xev.state = modifiers+0x10;
       xev.state = modifiers;
-      //xev.state = 0;
       xev.time = CurrentTime;
       xev.x = 1;
       xev.y = 1;
       xev.x_root = 1;
       xev.y_root = 1;
+      //xev.x = win_x_return;
+      //xev.y = win_y_return;
+      //xev.x_root = root_x_return;
+      //xev.y_root = root_y_return;
+      xev.same_screen = True;
         
       if (XSendEvent(dpy, w, True, KeyPressMask, ( XEvent* )&xev) == 0)
-         cerr << "XSendEvent failed." << endl;
-
-      xev.type = KeyRelease;
-      if (XSendEvent(dpy, w, True, KeyReleaseMask, ( XEvent* )&xev) == 0)
-         cerr << "XSendEvent failed." << endl;
+         error("XSendEvent failed.");
 
+      //xev.type = KeyRelease;
+      //if (XSendEvent(dpy, w, True, KeyReleaseMask, ( XEvent* )&xev) == 0)
+      //   cerr << "XSendEvent failed." << endl;
       
       XSync(dpy, false);
   }
@@ -303,7 +474,7 @@ void macroEAK_OPEN_TRAY(LCommand &command) {
                 if (macro == "EAK_EJECT" || macro == "EAK_OPEN_TRAY")
                    cd.openTray();
                 if (macro == "EAK_OPEN_TRAY_SCSI") {
-                   if (verbose) cout << "Calling the openTrayScsi() interface" << endl;
+                   vmsg("Calling the openTrayScsi() interface");
                    cd.openTrayScsi();
                 }
     }
@@ -321,7 +492,7 @@ void macroEAK_CLOSE_TRAY(LCommand &command) {
         string macro = command.getMacroType();
         if (args.size() == 0) {
            cdromCtrl myCdrom(myConfig->getValue("CdromDevice"));
-                   if (verbose) cout << "Calling the closeTray() interface" << endl;
+                   vmsg("Calling the closeTray() interface");
                    myCdrom.closeTray();
                 
         }
@@ -334,7 +505,7 @@ void macroEAK_CLOSE_TRAY(LCommand &command) {
                 if (default_Display != NULL) {
                    default_Display->show(string("Closing the CDROM tray"));
                 }
-                if (verbose) cout << "Calling the closeTray() interface" << endl;
+                vmsg("Calling the closeTray() interface");
                 cd.closeTray();
 
           }
@@ -445,9 +616,9 @@ void macroEAK_VOLUP(LCommand &command,int type) {
   string mixer = myConfig->getValue("MixerDevice");
   /** If there is just the macro, and no arguments, use the default volume stepping, and
       adjust by that */
-  if (verbose) cout << "EAK_VOLUP" << endl;
+  msg("EAK_VOLUP");
   if (args.size() == 0) {
-        if (verbose) cout << "doing default volume up\n";
+        msg("doing default volume up");
         /* Set the mixer device */
         soundCtrl mySound(mixer,type);
         retval = mySound.volumeUp();
@@ -459,7 +630,7 @@ void macroEAK_VOLUP(LCommand &command,int type) {
   // Assume we have just an increment value.
   if (args.size() == 1) {
       int value = 0;
-      if (verbose) cout << "single volume up\n";
+      msg("single volume up");
       value = atoi(args[0].c_str());
       soundCtrl mySound(mixer,type);
       retval = mySound.volumeUp(value);
@@ -469,7 +640,7 @@ void macroEAK_VOLUP(LCommand &command,int type) {
   // Assume we have integer and value pairs.
   if (args.size() >=2 && (args.size() % 2 == 0)) {
       int value = 0;
-      if (verbose) cout << "multiple volume ups\n";
+      msg("multiple volume ups");
       soundCtrl sound;
       for (vector<string>::const_iterator it = args.begin(); it != args.end();) {
           value = atoi(it->c_str());
@@ -493,7 +664,7 @@ void macroEAK_VOLDOWN(LCommand &command, int type) {
       adjust by that */
 
   if (args.size() == 0) {
-      if (verbose) cout << "default volumeDown\n";
+      msg("default volumeDown");
       /* Set the mixer device */
       soundCtrl mySound(mixer,type);
       retval = mySound.volumeDown();
@@ -539,33 +710,33 @@ void macroEAK_SLEEP(LCommand &command) {
         if (default_Display != NULL)
           default_Display->show ("Sleep");
 }
-/*
-void CommandExec::macroEAK_SCREEN_LOCK(LCommand &command) {
+
+void macroEAK_SCREEN_LOCK(LCommand &command) {
         const vector<string>& args = command.getArgs();
         if (!args.empty()) {
-    string method = args[0];
-    // Transform the method to uppercase so we can do a comparison.
-    int (*pf)(int)=toupper;
-    std::transform(method.begin(), method.end(), method.begin(), pf);
+    		string method = args[0];
+	    	// Transform the method to uppercase so we can do a comparison.
+    		int (*pf)(int)=toupper;
+    		std::transform(method.begin(), method.end(), method.begin(), pf);
                 string comm;
                 if (!fork()) {
                         if (method == "KDE")
                                 comm = "dcop kdesktop KScreensaverIface lock";
-                if (method == "GNOME" || method == "XSCREENSAVER" )
+                	if (method == "GNOME" || method == "XSCREENSAVER" )
                                 comm = "xscreensaver-command -lock";
 
-          //comm = lineak_util_functions::unescape(comm,"#");
-                                        comm += " &";
-                 // child process that tries to run the command, and then exits
-                // all specials done, let's go for it... ;)
-                if (verbose) cout << "Locking screen for desktop " << method << endl;
-                                        system(comm.c_str());
-                                        exit(1);
-                        }
-                        // NOTE: no, we don't wait() for the child. we just ignore the SIGCLD signal
+          		//comm = lineak_util_functions::unescape(comm,"#");
+                        comm += " &";
+                 	// child process that tries to run the command, and then exits
+                	// all specials done, let's go for it... ;)
+                	msg("Locking screen for desktop " + method);
+                                system(comm.c_str());
+                                exit(1);
+                 }
+                 // NOTE: no, we don't wait() for the child. we just ignore the SIGCLD signal
         }
         else
-                cerr << "EAK_SCREEN_LOCK macro requires an argument" << endl;
+                error("EAK_SCREEN_LOCK macro requires an argument");
 
 }
-*/
+
