@@ -154,20 +154,50 @@ void macroEAK_MediaDetect(LObject *obj, LCommand &command, XEvent xev) {
    map<int, ConfigDirectives*> heur;
    string macro = command.getMacroType();
    string home = getenv("HOME");
-   string conffile = home + LINEAKDIR;
+   string usrconffile = home + LINEAKDIR;
+   string sysconffile = string(CONFDIR) + "/";
    string arg = args[0];
    string carg = "";
-
+   
    if (args.size() == 2)
 	   carg = args[1];
 
-   conffile += "media-detect.conf";
+   usrconffile += "media-detect.conf";
+   sysconffile += "media-detect.conf";
 
+   map < string, ConfigDirectives* >* sysdefs = NULL;
    map < string, ConfigDirectives* >* defs = NULL;
-   MDLoader ldr(conffile);
-   defs = ldr.loadDef();
    ConfigDirectives *dir = NULL;
+
+   if (lineak_util_functions::file_exists(sysconffile)) {
+      MDLoader ldr(sysconffile);
+      sysdefs = ldr.loadDef();
+   }
+
+   if (lineak_util_functions::file_exists(usrconffile)) {
+      MDLoader ldr(usrconffile);
+      defs = ldr.loadDef();
+   }
+
+   // If there are use and system wide configurations merge them
+   // together.
+   if (sysdefs != NULL && defs != NULL) {
+      map < string, ConfigDirectives* >::iterator it = sysdefs->begin();
+      while (it != sysdefs->end()) {
+	  if ( defs->find(it->first) != defs->end() ) {
+	     free((*defs)[it->first]);
+	  }
+	  (*defs)[it->first] = it->second;
+	  it++;
+      }
+   }
+   //If sysdefs is not empty, but the user defs are the use the system configuration.
+   else if (defs == NULL && sysdefs != NULL)
+	   defs = sysdefs;
+   
+   if (defs != NULL ) {
    map < string, ConfigDirectives* >::iterator it = defs->begin();
+
    /** Determine which of the configured apps are running and
     *  add them to a list heur of the running programs. From
     *  this list, look at the application priorities to try
@@ -192,6 +222,7 @@ void macroEAK_MediaDetect(LObject *obj, LCommand &command, XEvent xev) {
 	}
    }
    dir = NULL;
+
    /** heur should have a list of the programs that are running. We should determine
     *  which one to pick and then act accordingly. */
     if (heur.size() == 1) {
@@ -230,11 +261,7 @@ void macroEAK_MediaDetect(LObject *obj, LCommand &command, XEvent xev) {
 	    if (type == "macro" || type == "MACRO" )
 		dispatch_macro(obj, command, dir, xev);
     }
-
-    /** Rotate the key state if it's a toggleable key */
-//    if (obj->isToggle())
-//        obj->toggleState();
-
+    }
  }  
 }
 
@@ -276,10 +303,18 @@ void dispatch_dcop(LObject *obj, LCommand &command, ConfigDirectives *dir) {
 void dispatch_program(LObject *obj, LCommand &command, ConfigDirectives *dir) {
    const vector<string>& args = command.getArgs();
    string arg = args[0];
-   string comm = dir->getValue(PROGRAM) + " ";
+   string comm;
+   string prog = dir->getValue(PROGRAM) + " ";
+   string controller = dir->getValue(CONTROLLER) + " ";
+
+   if (controller == " ")
+	   comm = prog;
+   else
+	   comm = controller;
+   
    comm += dir->getValue(arg);
 
-   msg("Executing on command: " + arg);
+   msg("Executing on command: " + comm);
    //obj->getType() == SYM
    if (global_enable) {
       if (!fork()) {
@@ -288,7 +323,7 @@ void dispatch_program(LObject *obj, LCommand &command, ConfigDirectives *dir) {
          /* child process that tries to run the command, and then exits */
          /* all specials done, let's go for it... ;) */
          if (verbose) cout << "... running " << comm << endl;
-         if (verbose) cout << "... displaying " << comm << endl;
+        //if (verbose) cout << "... displaying " << comm << endl;
          system(comm.c_str());
          exit(true);
       }
@@ -342,6 +377,10 @@ void dispatch_macro(LObject *obj, LCommand &command, ConfigDirectives *dir, XEve
       execute(mdobj,xev); 
 
     /** Use the OSD to show the output */
+      /** We should not show the display here. Instead, we should allow
+       * the macro to handle the display itself as it may have contextual
+       * information to display that we cannot get here. */
+      /*
     if ( mediadetect_Display != NULL ) {
          if (dname == "" || dname == snull) {
              string lname = dir->getValue(NAME) + ": ";
@@ -350,7 +389,7 @@ void dispatch_macro(LObject *obj, LCommand &command, ConfigDirectives *dir, XEve
          }
          else
              mediadetect_Display->show(dname);
-    }
+    }*/
 
    }
 }
